<?php
function cat ($a1, $a2) {
  $res = array();
  foreach ($a1 as $e)
    $res[] = $e;
  foreach ($a2 as $e)
    $res[] = $e;
  return $res;
}
abstract class field {
  public $name;
  public $default = '';
  public $is_required = 1;
  public $type;
  public $description;
  public $is_index = 0;
  public $is_unique = 0;
  public $is_sql_input = 0;
  public $permissions = 'all';
  function sql_field_is_required () {
    if ($this->is_required)
      return ' NOT NULL';
    else
      return '';
  }
  function sql_field () {
    $field  = $this->name . ' ' . $this->type;
    $field .= $this->sql_field_is_required();
    return $field;
  }
  function sql_value ($value) {
    if ($this->is_sql_input)
      return $value;
    else
      return "'" . $value . "'";
  }
  function default_value () {
    if ($_POST[$this->name] == '')
      return $this->default;
    else
      return $_POST[$this->name];
  }
  public function init_field ($name, $type, $is_required, $default, $description, $is_index, $is_unique) {
    $this->name = $name;
    $this->default = $default;
    $this->type = $type;
    $this->description = $description;
    $this->is_sql_input = 0;
    $this->is_required = $is_required;
    // $this->is_required = 0;
    $this->is_index = $is_index;
    $this->is_unique = $is_unique;
  }
  public function set_permissions ($val) {
    $this->permissions = $val;
    return $this;
  }
  public function is_input_optional ($value) {
    return $this->is_required == FALSE AND $value == '';
  }
  public function check_required ($value) {
    return ($this->is_required == FALSE OR $value != '');
  }
  public function check ($value) {
    return $this->check_required($value);
  }
  public function process_input ($value) {
    return $value;
  }
  public function show ($dbc, $id, $value) {
    print $value . "\n";
  }
  
}
abstract class any_str_field extends field {
  public $size    = -1;
  public $maxsize = -1;
  public function init_str_field ($maxsize, $size) {
    $this->maxsize = $maxsize;
    if ($maxsize == -1) {
      $this->size = $size;
    } else {
      if ($size == -1)
	$this->size = $maxsize;
      else
	$this->size = $size;
    }
  }
  function input ($dbc) {
    if ($this->maxsize == -1) {
      print '<textarea name="' . $this->name . 
	'" cols="' . "50" . '" rows="' . "5" . '" wrap=PHYSICAL>' . 
	$this->default_value() . '</textarea>';
    } else
      print '<input type="text" name="' . $this->name . 
	'" size="' . $this->size . '" maxsize="' . $this->maxsize . 
	'" value="' . $this->default_value() . '" />';
  }
  function sql_field () {
    $field = $this->name;
    if ($this->maxsize == -1)
      $field .= ' TEXT';
    else 
      $field .=  ' VARCHAR(' . $this->maxsize . ')';
    $field .= $this->sql_field_is_required();
    return $field;
  }
}
class str_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=-1, $is_index=0, $is_unique=0, $size=-1) {
    if ($maxsize == -1)
      $this->type = "TEXT";
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, $this->type, $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
}
class password_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=40, $is_index=0, $is_unique=0, $size=20) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    $this->permissions = 'user';
    return $this;
  }
  function input ($dbc) {
    print '<input type="password" name="' . $this->name . 
      '[0]" size="' . $this->size . '" maxsize="' . $this->maxsize . 
      // '" value="' . $this->default_value() . 
      '" />';
    print '<b><font color="#00ff00"> again </font></b>';
    print '<input type="password" name="' . $this->name . 
      '[1]" size="' . $this->size . '" maxsize="' . $this->maxsize . 
      // '" value="' . $this->default_value() . 
      '" />';
  }
  public function check ($values) {
    $is_ok = eregi ("^[[:alnum:]]{4,16}$", $values[0]);
    if ($is_ok AND $values[0] == $values[1])
      return TRUE;
    else {
      return "must be alphanumeric of length between 4 and 16";
    }
  }
  public function process_input ($values) {
    return crypt($values[0]);
  }
}
class name_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=32, $is_index=0, $is_unique=0, $size=32) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($value) {
    if ($this->is_input_optional($value) OR (eregi ("^[[:alnum:]]+$", $value)))
      return TRUE;
    else
      return "must be alphanumeric with no spaces";
  }
}
class zip_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $is_index=0, $is_unique=0, $size=-1) {
    $this->init_str_field(10, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($value) {
    return $this->is_input_optional($value) OR (eregi ("^([0-9]{5})(-{1}[0-9]{4})?$", $value));
  }
}
class url_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=64, $is_index=0, $is_unique=0, $size=-1) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($value) {
    return $this->check_required($value);
  }
  public function show ($dbc, $id, $value) {
    if ($value != '')
      print '<a href="' . $this->value . '">' . $this->name . "<a>/n";
  }
}
class phone_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=16, $is_index=0, $is_unique=0, $size=-1) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($value) {
    return $this->is_input_optional($value) OR
      (eregi ("^([0-9]{3}-{1})?([0-9]{3}-{1}[0-9]{4})$", $value));
  }
}
class state_field extends any_str_field {
  public $states
    = array( 'massachusetts' => 'MA', 'california' => 'CA', 'new york' => 'NY' );
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=32, $is_index=0, $is_unique=0, $size=32) {
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    $this->size    = $size;
    $this->maxsize = 2;
    return $this;
  }
  public function check ($value) {
    if ($this->is_input_optional($value) OR (eregi ("^[[:alpha:]]{2,2}$", $value)))
      return TRUE;
    else
      return "must be two characters";
  }
  function input ($dbc) {
    print '<select name="' . $this->name . '" value="' . $this->default_value() . '">';
    foreach ($this->states as $n => $a) {
      print '<option value="' . $a . '">' . $n . '</option>';
    }
    print '</select>';
  }
}
class list_field extends any_str_field {
  public $table;
  public $field;
  public $backfield;
  public function __construct 
      ($name, $table, $backfield, $field, $is_required=1, $default='', $description='', 
       $is_index=0, $is_unique=0, $size=32) {
    $this->init_field($name, "NONE", $is_required, $default, $description, $is_index, $is_unique);
    $this->table       = $table;
    $this->field       = $field;
    $this->backfield   = $backfield;
    return $this;
  }
  function sql_field () {
    return -1;
  }
  public function check ($value) {
    return TRUE;
    // return $this->is_input_optional($value) OR (eregi ("^[[:num:]]+$", $value));
  }
  function show ($dbc, $id, $value) {
    $query = 'SELECT * FROM ' . $this->table . ' WHERE ' . $this->backfield . '="' . $id . '"';
    if ($r = mysql_query ($query, $dbc)) {
      while ($row = mysql_fetch_array ($r)) 
	print " " . $row[$this->field];
    } 
  }
  function process_input ($value) {
    return -1;
  }
  function input ($dbc) {
    $query = 'SELECT * FROM ' . $this->table;
    $next  = "loc=" . urlencode("row-edit.php?tab={$_GET['tab']}&id={$_GET['id']}");
    if ($r = mysql_query ($query, $dbc)) {
      while ($row = mysql_fetch_array ($r)) {
	print '<a href=row-edit.php?tab=' . $this->table . "&id={$row['id']}>" . $row[$this->field] . "</a> ";
	print '<em><a href=row-drop.php?tab=' . $this->table . 
	  "&id={$row['id']}&" . $next . ">drop</a></em> ";
      }
    } 
    print '<a href=row-add.php?tab=' . $this->table . '&' . $next . '>add</a>';
  }
}
class any_ptr_field extends any_str_field {
  public $table;
  public $field;
  public function check ($value) {
    return TRUE;
    // return $this->is_input_optional($value) OR (eregi ("^[[:num:]]+$", $value));
  }
}
class ptr_field extends any_ptr_field {
  public function __construct 
      ($name, $table, $field, $is_required=1, $default='', $description='', 
       $is_index=0, $is_unique=0, $size=32) {
    $this->init_field($name, "INT", $is_required, $default, $description, $is_index, $is_unique);
    $this->is_unsigned = TRUE;
    $this->size        = $size;
    $this->table       = $table;
    $this->field       = $field;
    return $this;
  }
  function input ($dbc) {
    print '<select name="' . $this->name . '" value="' . $this->default_value() . '">';
    $query = 'SELECT * FROM ' . $this->table;
    if ($r = mysql_query ($query, $dbc)) {
      while ($row = mysql_fetch_array ($r)) 
	print '<option value="' . $row['id'] . '">' . $row[$this->field] . '</option>';
    } 
    print '</select> ';
    print '<a href=row-add.php?tab=' . $this->table . '>add</a>';
  }
}
class symptr_field extends any_ptr_field {
  public function __construct 
      ($name, $table, $field, $is_required=1, $default='', $description='', 
       $maxsize=32, $is_index=0, $is_unique=0, $size=32) {
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    $this->is_unsigned = TRUE;
    $this->size        = $size;
    $this->maxsize     = $maxsize;
    $this->table       = $table;
    $this->field       = $field;
    return $this;
  }
  function input ($dbc) {
    print '<select name="' . $this->name . '" value="' . $_POST[$this->name] . '">';
    $query = 'SELECT * FROM ' . $this->table;
    if ($r = mysql_query ($query, $dbc)) {
      while ($row = mysql_fetch_array ($r)) {
	if ($_POST[$this->name] == $row[$this->field])
	  $id = $row['id'];
	print '<option value="' . $row[$this->field] . '">' . $row[$this->field] . '</option>';
      }
    } 
    print '</select> ';
    print '<a href=row-edit.php?tab=' . $this->table . "&id=$id>edit</a> ";
    print '<a href=row-add.php?tab=' . $this->table . '>add</a>';
  }
}
class username_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=32, $is_index=0, $is_unique=0, $size=32) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($value) {
    if ($this->is_input_optional($value) OR (eregi ("^[[:alnum:]]+$", $value)))
      return TRUE;
    else
      return "must be alphanumeric with no spaces";
  }
}
class pic_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=100, $is_index=0, $is_unique=0, $size=32) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($value) {
    // return $this->is_input_optional($value) OR (eregi ("^[[:alnum:]]+$", $value));
    return TRUE;
  }
  function input ($dbc) {
    // print $_POST[$this->name] . "\n";
    $this->show(-1, 0, $_POST[$this->name]);
    print '<input type="file" name="' . $this->name . 
      '" size="' . $this->size . '" maxsize="' . $this->maxsize . 
      '" value="' . $_POST[$this->name] . '" />';
  }
  function process_input ($value) {
    $value = $_FILES[$this->name]['name'];
    if ($value == '') {
      return -1;
    } else {
      $upload_dir       = 'pics/'; 
      $base_upload_file = $upload_dir . $value;
      $upload_file      = $base_upload_file;
      for ($i=2; $i<1000; $i++) {
	if (file_exists($upload_file))
	  $upload_file = $base_upload_file . "-" . $i;
	else
	  break;
      }
      if (move_uploaded_file($_FILES[$this->name]['tmp_name'], $upload_file)) {
	// print "File is valid, and was successfully uploaded. ";
	// print "Here's some more debugging info:\n";
	// print_r($_FILES);
      } else {
	// print "Possible file upload attack!  Here's some debugging info:\n";
	// print_r($_FILES);
      }
      return $upload_file;
    }
  }
  public function show ($dbc, $id, $value) {
    if ($value != '')
      // print '<br /><img src="' . $value . "\"><br />\n";
      print '<img src="' . $value . "\">\n";
  }
}
class email_field extends any_str_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=50, $is_index=0, $is_unique=0, $size=-1) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "STR", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  public function check ($name) {
    return $this->is_input_optional($value) OR
      (eregi("^([[:alnum:]]|_|\.|-)+@([[:alnum:]]|\.|-)+(\.)([a-z]{2,4})$", $name));
  }
}
class any_int_field extends any_str_field {
  public $is_unsigned = 0;
  public $is_auto_increment = 0;
  function input ($dbc) {
    print '<input type="text" name="' . $this->name . 
      '" size="' . $this->size . '" maxsize="' . $this->maxsize . 
      '" value="' . $this->default_value() . '" />';
  }
  function sql_field () {
    $field = $this->name . ' INT';
    if ($this->is_unsigned) 
      $field .= ' UNSIGNED';
    if ($this->is_auto_increment) 
      $field .= ' AUTO_INCREMENT';
    $field .= $this->sql_field_is_required();
    return $field;
  }
}
class int_field extends any_int_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $maxsize=-1, $is_index=0, $is_unique=0, $is_auto_increment=0, $size=-1) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "INT", $is_required, $default, $description, $is_index, $is_unique);
    $this->is_unsigned       = 0;
    $this->is_auto_increment = $is_auto_increment;
    return $this;
  }
}
class uint_field extends any_int_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', 
       $is_index=0, $is_unique=0, $is_auto_increment=0) {
    $this->init_str_field($maxsize, $size);
    $this->init_field($name, "INT", $is_required, $default, $description, $is_index, $is_unique);
    $this->is_unsigned       = 1;
    $this->is_auto_increment = $is_auto_increment;
    return $this;
  }
}
class any_date_field extends field {
  public $months
    = array( 1 => 'January', 2 => 'February', 3 => 'March', 4 => 'April', 5 => 'May',
	     6 => 'June', 7 => 'July', 8 => 'August', 9 => 'September', 10 => 'October',
	     11 => 'November', 12 => 'December' );
  public function is_supplied ($value) {
    $is_supplied = TRUE;
    foreach ($value as $v)
      if ($v == '') $is_supplied = FALSE;
  }
  public function checkdate ($date) {
    return is_supplied($date) AND checkdate($date[0], $date[1], $date[2]);
  }
  public function is_input_optional ($value) {
    return $this->is_required == FALSE AND $this->is_supplied($value) == FALSE;
  }
  public function input_date ($dbc) {
    $date = $this->default_value();
    if ($date != '') {
      // year-mo-dy
      // 0123456789
      $year  = substr($date, 0, 4);
      $month = substr($date, 5, 2);
      $day   = substr($date, 8, 2);
      sscanf($month, "%d", $month_num);
      $month_name = $this->months[$month_num];
      // print 'YEAR ' . $year . ' MONTH ' . $month . ' ' . $month_name . ' DAY ' . $day . "\n";
    } else {
      $year = ''; $month = ''; $day = '';
    }
    print '<select name="' . $this->name . '[0]">';
    if ($month != '')
      print '<option value="' . $month . '">' . $month_name . '</option>';
    else
      print '<option value="">month</option>';
    foreach ($this->months as $d => $n) {
      print '<option value="' . $d . '">' . $n . '</option>';
    }
    print '</select>';
    print '<select name="' . $this->name . '[1]">';
    if ($month != '')
      print '<option value="' . $day . '">' . $day . '</option>';
    else
      print '<option value="">day</option>';
    for ($i = 1; $i <= 31; $i++) {
      print '<option value="' . $i . '">' . $i . '</option>';
    }
    print '</select>';
    print '<select name="' . $this->name . '[2]">';
    if ($year != '') 
      print '<option value="' . $year . '">' . $year . '</option>';
    else
      print '<option value="">year</option>';
    for ($i = 2005; $i > 1900; $i--) {
      print '<option value="' . $i . '">' . $i . '</option>';
    }
    print '</select>';
  }
  public function input_time ($dbc, $offset, $start) {
    $time = $this->default_value();
    if ($time != '') {
      // hr:mn:sc
      // 0123456789
      $hour   = substr($time, $offset + 0, 2);
      $minute = substr($time, $offset + 3, 2);
    } else {
      $hour = ''; $minute = '';
    }
    print '<select name="' . $this->name . '[' . ($start+0) . ']">';
    if ($hour != '')
      print '<option value="' . $hour . '">' . $hour . '</option>';
    else
      print '<option value="">hour</option>';
    for ($i = 0; $i <= 24; $i++) {
      print '<option value="' . $i . '">' . $i . '</option>';
    }
    print '</select>';
    print '<select name="' . $this->name . '[' . ($start+1) . ']">';
    if ($minute != '')
      print '<option value="' . $minute . '">' . $minute . '</option>';
    else
      print '<option value="">minute</option>';
    for ($i = 0; $i < 60; $i++) {
      print '<option value="' . $i . '">' . $i . '</option>';
    }
    print '</select>';
  }
}
class creation_date_field extends any_date_field {
  public function __construct
      ($name, $is_required=1, $default='', $description='', $is_index=0, $is_unique=0) {
    $this->init_field($name, "DATETIME", $is_required, $default, $description, $is_index, $is_unique);
    $this->permissions = 'admin';
    $this->is_sql_input = 1;
    return $this;
  }
  function input ($dbc) {
    if (isset($_POST[$this->name]))
      print '<em>' . $_POST[$this->name] . '</em>';
    else
      print '<em>now</em>';
    print '<input name=' . $this->name . ' type=hidden value="' . $_POST[$this->name] . '">';
  }
  public function process_input ($date) {
    if ($date != '')
      return "'" . $date . "'";
    else
      return 'NOW()';
  }
  function check ($value) {
    return TRUE;
  }
}
class now_field extends any_date_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', $is_index=0, $is_unique=0) {
    $this->init_field($name, "DATETIME", $is_required, $default, $description, $is_index, $is_unique);
    $this->permissions = 'admin';
    $this->is_sql_input = 1;
    return $this;
  }
  function input ($dbc) {
    if (isset($_POST[$this->name]))
      print '<em>' . $_POST[$this->name] . '</em>';
    else
      print '<em>now</em>';
    print '<input name=' . $this->name . ' type=hidden value="NOW()">';
  }
  function check ($value) {
    return TRUE;
  }
}
class date_field extends any_date_field {
  public function __construct
      ($name, $is_required=1, $default='', $description='', $is_index=0, $is_unique=0) {
    $this->init_field($name, "DATE", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  function input ($dbc) {
    $this->input_date($dbc);
  }
  public function check ($date) {
    return $this->is_input_optional($date) OR $this->checkdate($date);
  }
  public function process_input ($date) {
    $res = $date[2] . "-" . $date[0] . "-" . $date[1];
    // print $res;
    return $res;
  }
}
class time_field extends any_date_field {
  public function __construct 
      ($name, $is_required=1, $default='', $description='', $is_index=0, $is_unique=0) {
    $this->init_field($name, "TIME", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  function input ($dbc) {
    $this->input_time($dbc, 0, 0);
  }
  public function check ($time) {
    return $this->is_input_optional($time); // OR checkdate ($time[0], $time[1]);
  }
  public function process_input ($date) {
    $res = $time[0] . ":" . $time[1] . ":" . "0";
    return $res;
  }
}
class datetime_field extends any_date_field {
  public function __construct
      ($name, $is_required=1, $default='', $description='', $is_index=0, $is_unique=0) {
    $this->init_field($name, "DATETIME", $is_required, $default, $description, $is_index, $is_unique);
    return $this;
  }
  function input ($dbc) {
    $this->input_date($dbc, 0);
    $this->input_time($dbc, 11, 3);
  }
  public function check ($date) {
    return $this->is_input_optional($date) OR $this->checkdate ($date);
  }
  public function process_input ($date) {
    $res  = $date[2] . ":" . $date[0] . ":" . $date[1] . " ";
    $res .= $date[3] . "-" . $date[4] . "-" . "0";
    return $res;
  }
}
class table {
  public $name;
  public $fields;
  public $list_fields;
  public $admin_fields;
  public $ordered_by_fieldname = "id";
  public $permissions = 'admin';
  public $user_fieldname = '';
  public $is_show_others = TRUE;
  public function __construct
      ($name, $fields, $list_fieldnames=-1, $permissions='admin', $user_fieldname='', $is_show_others=TRUE) {
    $this->name           = $name;
    $this->fields         = $fields;
    $this->permissions    = $permissions;
    $this->user_fieldname = $user_fieldname;
    $this->is_show_others = $is_show_others;
    if ($list_fieldnames == -1) {
      foreach ($fields as $f) 
	$this->list_fields[] = $f;
    } else {
      foreach ($list_fieldnames as $n) 
	$this->list_fields[] = $this->find_field($n);
    }
    return $this;
  }
  function find_field ($name) {
    foreach ($this->fields as $f) 
      if ($f->name == $name)
	return $f;
  }
  function input ($dbc, $kind, $action, $is_add, $msgs) {
    print "<h3><center>" . $kind . " " . $this->name . "<center></h3>";
    if (empty($msgs) == FALSE) {
      print '<center><font color="#ff0000">';
      foreach ($msgs as $msg) {
	print $msg . "<br \>\n";
      }
      print '</font></center><br />';
    }
    print '<form name="form" enctype="multipart/form-data" action="' . $action . '" method="post">' . "\n";
    print '<table width="100%">' . "\n";
    foreach ($this->fields as $f) {
      if ($f->permissions == 'all' OR ($f->permissions == 'user' AND $is_add) OR $_SESSION['is_admin']) {
	print '<tr>' . "\n";
	print '<td align="right"><b>';
	if ($f->is_required) print '<font color="#00ff00">';
	print $f->name;
	if ($f->is_required) print '</font>';
	print '</b>';
	if ($f->description != '')
	  print '<br /><em>' . $f->description . '</em>';
	print '</td>' . "\n";
	print '<td align="left">' . "\n";
	$f->input($dbc);
	print '</tr>' . "\n";
      }
    }
    print '<tr><td align="center" colspan="2"><input type="submit" name="submit" value="submit"><input type="reset" name="reset" value="reset"></td></tr>' . "\n";
    print '</table>' . "\n";
    print '</form>' . "\n";
  }
  function tab_create ($dbc) {
    $query = 'CREATE TABLE ' . $this->name . ' (';
    $query .= "id INT UNSIGNED NOT NULL AUTO_INCREMENT";
    $is_first = 0;
    foreach ($this->fields as $f) {
      $val = $f->sql_field();
      if ($val != -1) {
	if ($is_first) $is_first = 0; else $query .= ', ';
	$query .= $val;
      }
    }
    $query .= ', PRIMARY KEY (id))';
    // print $query . "\n";
    $r = mysql_query ($query, $dbc); 
    return '';
  }
  function tab_drop ($dbc) {
    $query = 'DROP TABLE ' . $this->name;
    $r = mysql_query ($query, $dbc); 
    return '';
  }
  function delete ($dbc) {
    $query = "DELETE FROM " . $this->name . " WHERE id={$_GET['id']} LIMIT 1";
    $r = mysql_query ($query, $dbc); 
    return '';
  }
  function check ($row) {
    $is_ok = TRUE;
    $msgs  = array();
    foreach ($this->fields as $f) {
      $val = $row[$f->name];
      $msg = $f->check($val);
      if (is_string($msg) OR $msg == FALSE) {
	if ($msg == FALSE)
	  $msg = $val . " is in bad format";
	$msgs[] = "<b>" . $f->name . "</b> <em>" . $msg . " -- please reenter</em>";
      }
    }
    return $msgs;
  }
  function convert_vals ($row) {
    $res = array();
    foreach ($this->fields as $f) {
      if ($f->is_tup) {
	$val = array();
	for ($i = 0; $i < $f->tup_len; $i++)
	  $val[$i] = $row[$f->name . '[' . $i . ']'];
      } else
	$val = $row[$f->name];
      $res[$f->name] = $val;
    }
    return $res;
  }
  function check_unique ($dbc, $vals) {
    $query = '';
    $first = TRUE;
    foreach ($this->fields as $f) {
      if ($f->is_unique) {
	if ($query == '') $query = "select * from " . $this->name . " where ";
	if ($first) $first = FALSE; else $query .= ',';
	$query .= $f->name . "='" . $vals[$f->name] . "'";
      }
    }
    if ($query == '') 
      return array();
    $r   = @mysql_query($query, $dbc);
    $row = @mysql_fetch_object($r);
    if ($row) {
      $msg = 'entry already taken: one of';
      foreach ($this->fields as $f) {
	if ($f->is_unique)
	  $msg .= " " . $f->name . "=" . $vals[$f->name];
      }
      return array($msg);
    } else
      return array();
  }
  function add ($dbc) {
    $vals = $this->convert_vals($_POST);
    $msgs = $this->check($vals);
    if (empty($msgs)) {
      $msgs = $this->check_unique($dbc, $vals);
      if (empty($msgs)) {
	$query = "INSERT INTO " . $this->name . " ";
	$is_first = TRUE;
	$names  = "("; $values = "(";
	foreach ($this->fields as $f) {
	  $val = $f->process_input($vals[$f->name]);
	  if ($val != -1) {
	    if ($is_first) $is_first = FALSE; else { $names .= ", "; $values .= ", "; }
	    $names  .= $f->name;
	    $values .= $f->sql_value($val);
	  }
	}
	$names  .= ")"; $values .= ")";
	$query  .= $names . " values " . $values;
	// print $query . "\n";
	$r = mysql_query ($query, $dbc) or die(mysql_error());
      }
    }
    return $msgs;
  }
  function edit_input ($dbc) {
    $query = "SELECT * FROM " . $this->name . " WHERE id='{$_GET['id']}'";
    if ($r = mysql_query ($query, $dbc)) {
      $row = mysql_fetch_array ($r);
      foreach ($this->fields as $f) {
	$_POST[$f->name] = $row[$f->name];
      }
    }
  }
  function edit ($dbc) {
    $vals = $this->convert_vals($_POST);
    $msgs = $this->check($vals);
    if (empty($msgs)) {
      $query = "UPDATE " . $this->name . " SET ";
      $is_first = TRUE;
      foreach ($this->fields as $f) {
	$val = $f->process_input($vals[$f->name]);
	if ($val != -1) {
	  if ($is_first) $is_first = FALSE; else $query .= ", ";
	  $query .= $f->name . "=" . $f->sql_value($val);
	}
      }
      $query .= " WHERE id='" . $_GET['id'] . "'";
      $r = mysql_query ($query, $dbc);
      // print $query . "\n";
      if (mysql_affected_rows() != 1) 
	return array("unable to perform update");
    }
    return $msgs;
  }
  function show ($dbc) {
    print "<h3><center> " . $this->name . " <center></h3>";
    $query = 'SELECT * FROM ' . $this->name . ' ORDER BY ' . $this->ordered_by_fieldname . ' DESC';
    if ($r = mysql_query ($query, $dbc)) {
      print '<center><table>' . "\n";
      while ($row = mysql_fetch_array ($r)) {
	$is_user = ($this->permissions == 'user' AND $row[$this->user_fieldname] == $_SESSION['username']);
	if ($this->is_show_others == TRUE OR $is_user == TRUE) {
	  foreach ($this->list_fields as $f) {
	    if ($f->permissions == 'all' OR $_SESSION['is_admin']) {
	      if ($row[$f->name] != '') {
		print '<tr><th align=right>' . $f->name . '</th><td align=left>';
		$f->show($dbc, $row['id'], $row[$f->name]);
		print '</td></tr>';
	      }
	    }
	  }
	  if ($_SESSION['is_admin'] OR $this->permissions == 'all' OR $is_user) {
	    print '<tr><td colspan=2 align=center>';
	    print "<a href=\"row-edit.php?tab={$this->name}&id={$row['id']}\">edit</a>\n";
	    print "<a href=\"row-drop.php?tab={$this->name}&id={$row['id']}\">drop</a>\n";
	    print '</td></tr>';
	  }
	  print "<tr></tr>\n";
	}
      }
      if ($this->permissions != 'admin' OR $_SESSION['is_admin']) {
	print '<tr></tr><tr><td colspan=2 align=center><b>tab</b> ';
	print "<a href=\"row-add.php?tab={$this->name}\">add</a>";
	if ($_SESSION['is_admin']) 
	print " <a href=\"tab-drop.php?tab={$this->name}\">drop</a> ";
      }
      print "</td></tr>\n";
      print "</table></center><br />\n";
    } else {
      if ($_SESSION['is_admin']) {
	print "<center>\n";
	print "<b>tab</b> <a href=\"tab-create.php?tab={$this->name}\">create</a>";
	print "</center> </p><br />\n";
      }
    }
    return '';
  }
}
?>
